#include <tfslib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "config.h"

void help ()
{
  printf ("Encrypt/Decrypt file on card\n");
  printf ("Usage: [args] inode_in_hex\n");
  printf ("-h\thelp\n");
  printf ("-d device\tuse USB device\n");
  printf ("-e\tencrypt\n");
  printf ("-7\t7bit alphabet (packed)\n");
  printf ("-p <password>\tpassword\n");
  printf ("-s\tstrong (also len and magic)\n");
  exit (0);
}

int main (int argc, char **argv)
{
  u32 i;
  u16 c;
  u8 nb;
  u16 pos;
  u8 buf[1000];
  u8 len;
  int result;
  u8 sort;
  u16 j;
  u8 *d_name;
  u8 path[1000] = { '/', '\0' };
  u8 name[1000];
  u16 name_end;
  u16 path_end;
  b32 f;
  Inode ino;
  u8 enc = 0;
  u8 *pswd = NULL;
  u8 key[16];
  u8 strong = 0;
  u8 b7 = 0;
  u8 *dev = USBDEVICE;

  do
  {
    result = getopt (argc, argv, "ep:7shd:");
    switch (result)
    {
      case 'd':
	dev = optarg;
	break;
      case 'e':
	enc = 1;
	break;
      case 's':
	strong = 1;
	break;
      case '7':
	b7 = 1;
	break;
      case 'p':
	pswd = optarg;
	break;
      case 'h':
	help ();
	break;
    }
  }
  while (result > 0);

  if (pswd == NULL)
  {
    fprintf (stderr, "Missing password\n");
    help ();
  }

  strncpy (key, pswd, 16);

  if (argc - optind != 1)
    help ();
  sscanf (argv[optind], "%x", &i);
  f = fs_ino_p (i >> 8, i & 0xFF);
//printf("DST %x %04X %02X\n", f, fs_ino_block(f), fs_ino_index(f));

  init_mc (dev);

  if (mc_detect () != MC_OK)
  {
    printf ("Memory card not formatted\n");
    exit (1);
  }

  get_inode (&ino, f);
  printf ("Inode: %04X%02X\n", fs_ino_block (f), fs_ino_index (f));

  if (enc == 1 && ino.type != INODE_TYPE_FILE)
  {
    fprintf (stderr, "Not a regular file\n");
    goto X1;
  }
  if (enc == 0
      && (ino.type != INODE_TYPE_PROTECTED_KEY_DATA
	  && ino.type != INODE_TYPE_PROTECTED_DATA_LEN_TYPE))
  {
    fprintf (stderr, "Not encrypted file\n");
    goto X1;
  }

  if (enc == 0 && ino.type == INODE_TYPE_PROTECTED_DATA_LEN_TYPE)
    strong = 1;

  if (enc == 0)
  {
    //decrypt
    nb = mc_file_decrypt (f, key);
    if (nb == ERR_WRONG_KEY)
    {
      fprintf (stderr, "Wrong key\n");
      goto X1;
    }
    if (nb != NO_ERROR)
    {
      fprintf (stderr, "ERROR %d\n", nb);
      goto X1;
    }
  }
  else
  {
    //encrypt
    if (strong)
      mc_file_encrypt (f, key, INODE_TYPE_PROTECTED_DATA_LEN_TYPE);
    else
      mc_file_encrypt (f, key, INODE_TYPE_PROTECTED_KEY_DATA);
  }

X1:
  close_mc ();
}
