#include <tfslib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "config.h"

void help ()
{
  printf ("Format memory card\n");
  printf ("Usage: [args]\n");
  printf ("-d device\tuse USB device\n");
  printf ("-c\tclean (only for -f)\n");
  printf ("-f\tformat\n");
  printf ("-r\trefresh only\n");
  printf ("-l <lbl>\tlabel\n");
  printf ("-h\thelp\n");
  printf ("\nEither -f or -r or -l must be set.\n");
  exit (0);
}

u8 *_def_label = "My Card 1";

int main (int argc, char **argv)
{
  b32 w;
  u16 a;
  u8 ai;
  u32 i;
  u8 act = 0;
  u8 *label = NULL;
  int result;
  u8 o_r = 0;
  u8 o_f = 0;
  u8 o_c = 0;
  u8 *dev = USBDEVICE;

  do
  {
    result = getopt (argc, argv, "crfl:hd:");
    switch (result)
    {
      case 'd':
	dev = optarg;
	break;
      case 'l':
	label = optarg;
	break;
      case 'c':
	o_c = 1;
	break;
      case 'r':
	o_r = 1;
	break;
      case 'f':
	o_f = 1;
	break;
      case 'h':
	help ();
	break;
    }
  }
  while (result > 0);

  if (o_c == 0 && o_r == 0 && label == NULL && o_f == 0)
    help ();
  if (o_r == 1 && o_f == 1)
    help ();
  if (o_c == 1 && o_f == 0)
    help ();

  init_mc (dev);

  if (o_c)
  {
    printf ("Cleaning...\n");
    for (i = 0; i < mc_pages () / 256; i++)
    {
      printf ("Cleaning block=%d/(0-%d)\r", i, mc_pages () / 256 - 1);
      //dataFlashBlockErase(i);
      dataFlashPageErase (i);
    }
    mc_detect ();
  }

  if (o_f)
    format ();

  if (o_r)
  {
    u8 zr[4];

    zr[0] = zr[1] = zr[2] = zr[3] = 0;

    for (i = 0; i < mc_pages (); i++)
    {
      printf ("Refreshing page=%d/(0-%d)\r", i, mc_pages () - 1);
      dataFlashMemoryToBufferRead (0, i);
      dataFlashBufferWrite (0, zr, mc_blocks () * 264 - 4, 4);
      dataFlashBufferToMemoryProgramWithErase (0, i);
    }
    mc_detect ();
  }

//__df_print(0);

  if (o_f || label)
  {
    if (label == NULL)
      label = _def_label;
    {
      u8 l[15];

      strncpy (l, label, 15);
      mc_set_label (l);
    }
  }
  printf ("\n");
  close_mc ();
}
