#include <tfslib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "config.h"

void help ()
{
  printf ("List directory\n");
  printf ("Usage: [args] [dir]\n");
  printf ("-d device\tuse USB device\n");
  printf ("-e\tprint ext\n");
  printf ("-l\tuse a long listing format, same -s -e -t\n");
  printf ("-R\trecursive\n");
  printf ("-s\tprint size\n");
  printf ("-t\tprint type\n");
  printf ("-h\thelp\n");
  exit (0);
}

u8 o_r = 0;
u8 o_e = 0;
u8 o_t = 0;
u8 o_s = 0;

void print_ino (b32 f, Inode * ino);

void print_dir (b32 w)
{
  u8 ai, start_i;
  u16 a, start;
  Inode ino;
  u32 dirs[10000];
  u32 nr_dirs = 0;
  u32 i;

//printf("ROOT %04X%02X\n", fs_ino_block(root()), fs_ino_index(root()));

  if (o_r);			//print path:\n
  {
    u8 pth[1000];
    u8 l;
    u16 p = 0;
    u32 pr = w;

    if (pr == root ())
      printf ("/");
    else
      while (pr != root ())
      {
	get_inode (&ino, pr);
	l = strlen (ino.text);
	for (i = 0; i < l; i++)
	{
	  pth[p++] = ino.text[l - 1 - i];
	}
	pth[p++] = '/';
	pr = fs_ino_p (ino.parent, ino.parent_index);
      }

    for (i = 0; i < p; i++)
      printf ("%c", pth[p - i - 1]);
    printf (":\n");
  }

  get_inode (&ino, w);
  start = a = ino.content;
  start_i = ai = ino.content_index;

//printf("XXXA %d AI %d\n", a, ai);

  if (a)
    do
    {
//printf("XXXA %d AI %d\n", a, ai);
//      df_print(a);
      get_inode (&ino, fs_ino_p (a, ai));
      if (ino.type != INODE_TYPE_UNUSED)
      {
	if (o_r)
	  if (ino.type == INODE_TYPE_DIR)
	    dirs[nr_dirs++] = fs_ino_p (a, ai);
//      printf("X_4\n");
//      for(i=0;i<sizeof(Inode);i++)printf("%02X ",((u8*)ino)[i]);
//      printf("X_5\n");
	print_ino (fs_ino_p (a, ai), &ino);
      }
      a = ino.next;
      ai = ino.next_index;
    }
    while (!(a == start && ai == start_i));
  printf ("\n");

  if (o_r)
  {
    for (i = 0; i < nr_dirs; i++)
      print_dir (dirs[i]);
  }

}

void print_ino (b32 f, Inode * ino)
{
  u32 i, j;
  u8 ext[3];

  printf ("%04X%02X ", fs_ino_block (f), fs_ino_index (f));

  for (i = 0; i < INODE_TEXT_LEN - 1; i++)
    if (i < strlen (ino->text))
      printf ("%c", ino->text[i]);
    else
      printf (" ");
  printf ("\t");
  if (o_t)
  {
    switch (ino->type)
    {
      case INODE_TYPE_UNUSED:
	printf ("unused\t");
	break;
      case INODE_TYPE_FILE:
	printf ("file\t");
	break;
      case INODE_TYPE_DIR:
	printf ("dir\t");
	break;
      case INODE_TYPE_PROTECTED_KEY_DATA:
	printf ("prot_key_data\t");
	break;
      case INODE_TYPE_PROTECTED_DATA_LEN_TYPE:
	printf ("prot_data_len_type\t");
	break;
      default:
	printf ("unknown\t");
	break;
    }

  }
  if (o_e)
  {
    for (i = 0; i < 3; i++)
      ext[i] = 0;
    mc_get_type (f, ext);
    for (i = 0; i < 3; i++)
      if (ext[i] >= 0x20 && ext[i] < 0x7F)
	printf ("%c", ext[i]);
      else
	printf ("?");

    printf (" ");
  }

  if (o_s)
    printf ("%d ", get_len (f));

  printf ("\n");
}

int main (int argc, char **argv)
{
  b32 w;
  int result;
  u8 *dev = USBDEVICE;

  do
  {
    result = getopt (argc, argv, "shelRtd:");
    switch (result)
    {
      case 'd':
	dev = optarg;
	break;
      case 's':
	o_s = 1;
	break;
      case 'e':
	o_e = 1;
	break;
      case 't':
	o_t = 1;
	break;
      case 'l':
	o_s = 1;
	o_e = 1;
	o_t = 1;
	break;
      case 'R':
	o_r = 1;
	break;
      case 'h':
	help ();
	break;
    }
  }
  while (result > 0);

  init_mc (dev);

  if (mc_detect () != MC_OK)
  {
    printf ("Memory card not formatted\n");
    exit (1);
  }

  if (argc - optind == 1)
    w = cd (argv[optind++]);
  else
    w = root();

  if (w < 0)
  {
    fprintf (stderr, "Path %s does not exist\n", optarg);
    exit (1);
  }

  print_dir (w);
  printf ("Free %d/%d blocks\n", mc_free (), mc_pages () * mc_blocks ());

  close_mc ();
}
