#include <tfslib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "config.h"

void help ()
{
  printf ("Display inode status\n");
  printf ("Usage: [args] inode_in_hex\n");
  printf ("-h\thelp\n");
  printf ("-d device\tuse USB device\n");
  printf ("-p\tdump\n");
  exit (0);
}

void print_file (b32 f, Inode * ino)
{
  u32 i, j;
  u8 ext[3];

  printf ("Type: Regular File\n");
  printf ("Name_len: %02X\n", strlen (ino->text));
  printf ("Name: ");
  for (i = 0; i < strlen (ino->text); i++)
    printf ("%c", ino->text[i]);
  printf ("\n");
  printf ("Path: ");
  {
    u8 pth[1000];
    u8 l;
    u16 p = 0;
    u32 pr = f;

    while (pr != root ())
    {
      get_inode (ino, pr);
      l = strlen (ino->text);
      for (i = 0; i < l; i++)
      {
	pth[p++] = ino->text[l - 1 - i];
      }
      pth[p++] = '/';
      pr = fs_ino_p (ino->parent, ino->parent_index);
    }

    for (i = 0; i < p; i++)
      printf ("%c", pth[p - i - 1]);
    printf ("\n");
  }

  printf ("Size: %u\n", get_len (f));
  mc_get_type (f, ext);
  printf ("Ext: ");
  for (i = 0; i < 3; i++)
    if (ext[i] >= 0x20 && ext[i] < 0x7F)
      printf ("%c", ext[i]);
    else
      printf ("?");
  printf ("\n");
}

void print_dir (b32 f, Inode * ino)
{
//nr of inodes
}

void print_prot_key (b32 f, Inode * ino)
{
}

void print_prot_data (b32 f, Inode * ino)
{
}

int main (int argc, char **argv)
{
  u32 i;
  u16 c;
  u8 nb;
  u16 pos;
  u8 buf[1000];
  u8 len;
  int result;
  u8 sort;
  u16 j;
  u8 *d_name;
  u8 path[1000] = { '/', '\0' };
  u8 name[1000];
  u16 name_end;
  u16 path_end;
  b32 f;
  u8 dump = 0;
  Inode ino;
  u8 *dev = USBDEVICE;

  do
  {
    result = getopt (argc, argv, "phd:");
    switch (result)
    {
      case 'p':
	dump = 1;
	break;
      case 'h':
	help ();
	break;
    }
  }
  while (result > 0);

  if (argc - optind != 1)
    help ();
  sscanf (argv[optind], "%x", &i);
  f = fs_ino_p (i >> 8, i & 0xFF);
//printf("DST %x %04X %02X\n", f, fs_ino_block(f), fs_ino_index(f));

  init_mc (dev);
  if (mc_detect () != MC_OK)
  {
    printf ("Memory card not formatted\n");
    exit (1);
  }

  get_inode (&ino, f);
  printf ("Inode: %04X%02X\n", fs_ino_block (f), fs_ino_index (f));

  switch (ino.type)
  {
    case INODE_TYPE_UNUSED:
      printf ("Type: unused\n");
      break;
    case INODE_TYPE_FILE:
      print_file (f, &ino);
      break;
    case INODE_TYPE_DIR:
      print_dir (f, &ino);
      break;
    case INODE_TYPE_PROTECTED_KEY_DATA:
      print_prot_key (f, &ino);
      break;
    case INODE_TYPE_PROTECTED_DATA_LEN_TYPE:
      print_prot_data (f, &ino);
      break;
    default:
      printf ("Type: unknown\n");
      break;
  }

X1:
  close_mc ();
}
